package repository

import (
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/db/model"
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/db/util"
	"gorm.io/gorm"
)

type Chat struct {
	db *gorm.DB
}

func NewChat(db *gorm.DB) *Chat {
	return &Chat{db: db}
}

func (c *Chat) ByID(id uint64) (*model.Chat, error) {
	var chat model.Chat
	if err := c.db.Model(&chat).First(&chat, id).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &chat, nil
}

func (c *Chat) ByTelegramID(id int64) (*model.Chat, error) {
	var chat model.Chat
	if err := c.db.Model(model.Chat{TelegramID: id}).First(&chat).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &chat, nil
}

func (c *Chat) ByTelegramIDWithIntegrations(id int64) (*model.Chat, error) {
	var chat model.Chat
	if err := c.db.Model(model.Chat{TelegramID: id}).Preload("Integrations").First(&chat).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &chat, nil
}

func (c *Chat) ByIDWithIntegrations(id uint64) (*model.Chat, error) {
	var chat model.Chat
	if err := c.db.Model(&chat).Preload("Integrations").First(&chat, id).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &chat, nil
}

func (c *Chat) Save(chat *model.Chat) error {
	return c.db.Model(chat).Save(chat).Error
}

func (c *Chat) Delete(chat *model.Chat) error {
	return c.db.Model(&model.Chat{ID: chat.ID}).Delete(chat).Error
}
