package repository

import (
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/db/model"
	"gitea.neur0tx.site/Neur0toxine/vegapokerbot/internal/db/util"
	"gorm.io/gorm"
)

type User struct {
	db *gorm.DB
}

func NewUser(db *gorm.DB) *User {
	return &User{db: db}
}

func (u *User) ByID(id uint64) (*model.User, error) {
	var user model.User
	if err := u.db.Model(&user).First(&user, id).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &user, nil
}

func (u *User) ByTelegramID(id int64) (*model.User, error) {
	var user model.User
	if err := u.db.Model(model.User{TelegramID: id}).First(&user).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &user, nil
}

func (u *User) ByTelegramIDs(ids []int64) ([]model.User, error) {
	var users []model.User
	if err := u.db.Model(model.User{}).Where("telegram_id in (?)", ids).Find(&users).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return users, nil
}

func (u *User) ByIDWithChats(id uint64) (*model.User, error) {
	var user model.User
	if err := u.db.Model(&user).Preload("Chats").First(&user, id).Error; err != nil {
		return nil, util.HandleRecordNotFound(err)
	}
	return &user, nil
}

func (u *User) Save(user *model.User) error {
	return u.db.Model(user).Save(user).Error
}
